package com.qmth.themis.business.templete.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import com.qmth.themis.business.bean.admin.OpenRecordAnswerBean;
import com.qmth.themis.business.bean.admin.OpenRecordAnswerStructBean;
import com.qmth.themis.business.bean.admin.OpenRecordAnswerTempBean;
import com.qmth.themis.business.bean.admin.OpenRecordNeedMarkBean;
import com.qmth.themis.business.bean.cloudmark.FileUploadParams;
import com.qmth.themis.business.bean.cloudmark.SaveExamParams;
import com.qmth.themis.business.bean.cloudmark.SaveStudentParams;
import com.qmth.themis.business.bean.cloudmark.SaveSubjectParams;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.ExamCacheBean;
import com.qmth.themis.business.cache.bean.ExamCourseCacheBean;
import com.qmth.themis.business.cache.bean.ExamPaperCacheBean;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dao.TEExamCourseMapper;
import com.qmth.themis.business.dto.ExamStudentImportDto;
import com.qmth.themis.business.dto.RoomCodeImportDto;
import com.qmth.themis.business.entity.*;
import com.qmth.themis.business.enums.CloudMarkFileUploadTypeEnum;
import com.qmth.themis.business.enums.ExamModeEnum;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.enums.TaskStatusEnum;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.templete.service.TempleteLogicService;
import com.qmth.themis.business.util.CloudMarkUtil;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.OssUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Base64Util;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Description: 导入导出模版处理逻辑 impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/31
 */
@Service
public class TempleteLogicServiceImpl implements TempleteLogicService {
    private final static Logger log = LoggerFactory.getLogger(TempleteLogicServiceImpl.class);

    @Resource
    TEExamStudentService teExamStudentService;

    @Resource
    TEStudentService teStudentService;

    @Resource
    TBExamInvigilateUserService tbExamInvigilateUserService;

    @Resource
    TBUserService tbUserService;

    @Resource
    TEExamCourseService teExamCourseService;

    @Resource
    TEExamCourseMapper teExamCourseMapper;

    @Resource
    TBUserRoleService tbUserRoleService;

    @Resource
    TBExamInvigilateUserTempService tbExamInvigilateUserTempService;

    @Resource
    CloudMarkUtil cloudMarkUtil;

    @Resource
    TEExamService teExamService;

    @Resource
    TEExamPaperService teExamPaperService;

    @Resource
    CacheService cacheService;

    @Resource
    TOeExamRecordService tOeExamRecordService;

    @Resource
    TBTaskHistoryService tbTaskHistoryService;

    @Resource
    OssUtil ossUtil;

    @Resource
    TOeExamAnswerService examAnswerService;

    /**
     * 考生导入逻辑
     *
     * @param finalList
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> execImportExamStudentLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) throws UnsupportedEncodingException {
        Long orgId = Long.parseLong(String.valueOf(map.get(SystemConstant.ORG_ID)));
        Long examId = Long.parseLong(String.valueOf(map.get(SystemConstant.EXAM_ID)));
        Long createId = Long.parseLong(String.valueOf(map.get(SystemConstant.CREATE_ID)));
        List<String> txtList = (List<String>) map.get("txtList");
        TBTaskHistory tbTaskHistory = (TBTaskHistory) map.get("tbTaskHistory");
        List<TEExamStudent> teExamStudentList = new ArrayList<>();
        List<TEStudent> teStudentList = new ArrayList<>();
        List<TBExamInvigilateUser> tbExamInvigilateUserList = new ArrayList<>();
//        List<TBExamInvigilateUserTemp> tbExamInvigilateUserTempList = new ArrayList<>();
        Gson gson = new Gson();
        Set<String> roomCodeAndNameSet = new HashSet<>();
        Map<String, TEExamActivity> teExamActivityMap = (Map<String, TEExamActivity>) map.get("teExamActivityMap");
        Map<String, String> courseCodeMap = (Map<String, String>) map.get("courseCodeMap");
        QueryWrapper<TEExamCourse> teExamCourseQueryWrapper = new QueryWrapper<>();
        Set<String> courseCodeSet = courseCodeMap.keySet().stream().collect(Collectors.toSet());
        teExamCourseQueryWrapper.lambda().in(TEExamCourse::getCourseCode, courseCodeSet).eq(TEExamCourse::getExamId, examId);
        List<TEExamCourse> teExamCourseList = teExamCourseService.list(teExamCourseQueryWrapper);
        if (Objects.nonNull(teExamCourseList) && teExamCourseList.size() > 0) {
            teExamCourseList.forEach(s -> {
                courseCodeMap.remove(s.getCourseCode());
            });
        }
        List<TEExamCourse> teExamCourses = new ArrayList<>();
        courseCodeMap.forEach((k, v) -> {
            TEExamCourse teExamCourse = new TEExamCourse(examId, k, v, createId);
            teExamCourses.add(teExamCourse);
        });
        if (teExamCourses.size() > 0) {
            teExamCourseMapper.insertBatch(teExamCourses);
        }
        TEExamActivity teExamActivity = null;
        ExamModeEnum modeEnum = ExamModeEnum.valueOf(String.valueOf(map.get("mode")));
        if (Objects.equals(modeEnum, ExamModeEnum.ANYTIME)) {
            Iterator iterator = teExamActivityMap.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry<String, Map> entry = (Map.Entry<String, Map>) iterator.next();
                Map m = entry.getValue();
                teExamActivity = gson.fromJson(gson.toJson(m), TEExamActivity.class);
            }
        }
        int line = 0;
        //保存用户
        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> finalMap = finalList.get(i);
            List<Object> examStudentImportDtoList = finalMap.get(i);
            int min = 0;
            int max = SystemConstant.MAX_IMPORT_SIZE, size = examStudentImportDtoList.size();
            if (max >= size) {
                max = size;
            }
            while (max <= size) {
                List subList = examStudentImportDtoList.subList(min, max);
                for (int y = 0; y < subList.size(); y++) {
                    line++;
                    ExamStudentImportDto examStudentImportDto = (ExamStudentImportDto) subList.get(y);
                    if (Objects.equals(modeEnum, ExamModeEnum.TOGETHER)) {
                        Map m = (Map) teExamActivityMap.get(examStudentImportDto.getExamActivityCode());
                        teExamActivity = gson.fromJson(gson.toJson(m), TEExamActivity.class);
                    }
                    //先根据证件号+科目代码查询考生是否存在，存在则更新，不存在则插入
                    QueryWrapper<TEExamStudent> teExamStudentQueryWrapper = new QueryWrapper<>();
                    teExamStudentQueryWrapper.lambda().eq(TEExamStudent::getIdentity, examStudentImportDto.getIdentity())
                            .eq(TEExamStudent::getCourseCode, examStudentImportDto.getCourseCode())
                            .eq(TEExamStudent::getExamId, examId)
                            .eq(TEExamStudent::getExamActivityId, teExamActivity.getId());
                    TEExamStudent teExamStudent = teExamStudentService.getOne(teExamStudentQueryWrapper);
                    //如果为空则插入考生数据，插入考生前先插入学生档案数据
                    QueryWrapper<TEStudent> teStudentQueryWrapper = new QueryWrapper<>();
                    teStudentQueryWrapper.lambda().eq(TEStudent::getIdentity, examStudentImportDto.getIdentity()).eq(TEStudent::getOrgId, orgId);
                    TEStudent teStudent = teStudentService.getOne(teStudentQueryWrapper);
                    if (Objects.isNull(teExamStudent)) {
                        if (Objects.isNull(teStudent)) {//如果学生数据为空则插入学生数据
                            //先插入学生档案数据
                            teStudent = new TEStudent(orgId, examStudentImportDto.getIdentity(), examStudentImportDto.getName(), createId);
                        }
                        teExamStudent = gson.fromJson(gson.toJson(examStudentImportDto), TEExamStudent.class);
                        teExamStudent.setIdentity(examStudentImportDto.getIdentity());
                        teExamStudent.setExamId(examId);
                        teExamStudent.setStudentId(teStudent.getId());
                        teExamStudent.setCreateId(createId);
                    } else {
                        teExamStudent.setUpdateId(createId);
                        teExamStudent.setName(examStudentImportDto.getName());
                        teExamStudent.setGrade(examStudentImportDto.getGrade());
                        teExamStudent.setClassNo(examStudentImportDto.getClassNo());
                        teExamStudent.setRoomCode(examStudentImportDto.getRoomCode());
                        teExamStudent.setRoomName(examStudentImportDto.getRoomName());
                    }
                    teExamStudent.setCourseCode(examStudentImportDto.getCourseCode());
                    teExamStudent.setCourseName(examStudentImportDto.getCourseName());
                    teExamStudent.setAlreadyExamCount(0);
                    teExamStudent.setExamActivityId(teExamActivity.getId());
                    if (Objects.nonNull(examStudentImportDto.getPassword()) && !Objects.equals(examStudentImportDto.getPassword(), "")) {
                        teStudent.setPassword(Base64Util.encode(examStudentImportDto.getPassword().getBytes(SystemConstant.CHARSET_NAME)));
                    } else {
                        if (Objects.isNull(teStudent.getPassword())) {
                            teStudent.setPassword(SystemConstant.DEFAULT_PASSWORD);
                        }
                    }
                    teStudent.setMobileNumber(examStudentImportDto.getMobileNumber());
                    teStudent.setName(examStudentImportDto.getName());
                    teStudentList.add(teStudent);

                    ExamCourseCacheBean examCourseCacheBean = teExamCourseService.getExamCourseCacheBean(teExamStudent.getExamId(), teExamStudent.getCourseCode());
                    if (Objects.nonNull(examCourseCacheBean)) {
                        teExamStudent.setCourseName(examCourseCacheBean.getCourseName());
                    }
                    if (teExamStudent.getAlreadyExamCount() == null) {
                        teExamStudent.setAlreadyExamCount(0);
                    }
                    roomCodeAndNameSet.add(examId + ":" + teExamStudent.getRoomCode() + ":" + teExamStudent.getRoomName());
                    teExamStudentList.add(teExamStudent);
                }
                txtList = this.importProgress(max, min, size, txtList, tbTaskHistory);
                if (max == size) {
                    break;
                }
                min = max;
                max += SystemConstant.MAX_IMPORT_SIZE;
                if (max >= size) {
                    max = size;
                }
            }
            if (!CollectionUtils.isEmpty(teStudentList)) {
                teStudentService.saveOrUpdateBatch(teStudentList);
            }
            if (!CollectionUtils.isEmpty(teExamStudentList)) {
                teExamStudentService.saveOrUpdateBatch(teExamStudentList);
            }
        }
        map.put("line", line);
        //考场创建
        if (Objects.nonNull(roomCodeAndNameSet) && roomCodeAndNameSet.size() > 0) {
            Map<String, TBExamInvigilateUser> tbExamInvigilateUserMap = (Map<String, TBExamInvigilateUser>) map.get("tbExamInvigilateUserMap");
            AtomicInteger count = new AtomicInteger(0);
            roomCodeAndNameSet.forEach(s -> {
                if (Objects.isNull(tbExamInvigilateUserMap) || (Objects.nonNull(tbExamInvigilateUserMap) && Objects.isNull(tbExamInvigilateUserMap.get(s)))) {
                    String[] strs = s.split(":");
                    TBExamInvigilateUser tbExamInvigilateUser = new TBExamInvigilateUser(Long.parseLong(strs[0]), orgId, strs[1], strs[2]);
                    tbExamInvigilateUserList.add(tbExamInvigilateUser);

                    QueryWrapper<TBExamInvigilateUserTemp> tbExamInvigilateUserTempQueryWrapper = new QueryWrapper<>();
                    tbExamInvigilateUserTempQueryWrapper.lambda().eq(TBExamInvigilateUserTemp::getExamId, Long.parseLong(strs[0]))
                            .eq(TBExamInvigilateUserTemp::getOrgId, orgId)
                            .eq(TBExamInvigilateUserTemp::getRoomCode, strs[1])
                            .eq(TBExamInvigilateUserTemp::getRoomName, strs[2]);

                    map.put("dataRemark", "编码:" + strs[1] + ",名称:" + strs[2]);
                    TBExamInvigilateUserTemp tbExamInvigilateUserTemp = tbExamInvigilateUserTempService.getOne(tbExamInvigilateUserTempQueryWrapper);
                    tbExamInvigilateUserTemp = Objects.isNull(tbExamInvigilateUserTemp) ? new TBExamInvigilateUserTemp(Long.parseLong(strs[0]), orgId, strs[1], strs[2]) : tbExamInvigilateUserTemp;
                    tbExamInvigilateUserTempService.saveOrUpdate(tbExamInvigilateUserTemp);
                    count.getAndIncrement();
                }
            });
            if (!CollectionUtils.isEmpty(tbExamInvigilateUserList)) {
                tbExamInvigilateUserService.saveOrUpdateBatch(tbExamInvigilateUserList);
            }
            if (count.get() > 0) {
                txtList.add(DateUtil.format(new Date(), Constants.DEFAULT_DATE_PATTERN) + "->创建了" + count + "条考场数据");
            }
        }
        return map;
    }

    /**
     * 考场导入逻辑
     *
     * @param finalList
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> execImportRoomCodeLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) throws UnsupportedEncodingException {
        Long orgId = null;
        if (Objects.nonNull(map.get(SystemConstant.ORG_ID))) {
            orgId = Long.parseLong(String.valueOf(map.get(SystemConstant.ORG_ID)));
        }
        Long examId = Long.parseLong(String.valueOf(map.get(SystemConstant.EXAM_ID)));
        Long createId = Long.parseLong(String.valueOf(map.get(SystemConstant.CREATE_ID)));
        List<String> txtList = (List<String>) map.get("txtList");
        Set<TBUser> tbUserList = new HashSet<>();
        Set<TBUserRole> tbUserRoleList = new HashSet<>();
        Set<TBExamInvigilateUser> tbExamInvigilateUserList = new HashSet<>();
        TBTaskHistory tbTaskHistory = (TBTaskHistory) map.get("tbTaskHistory");
        Set<String> roomCodeSet = new HashSet<>();
        Set<String> roomNameSet = new HashSet<>();
        //保存用户
        int line = 0;
        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> finalMap = finalList.get(i);
            List<Object> examStudentDtoImportList = finalMap.get(i);
            int min = 0;
            int max = SystemConstant.MAX_IMPORT_SIZE, size = examStudentDtoImportList.size();
            if (max >= size) {
                max = size;
            }
            while (max <= size) {
                List subList = examStudentDtoImportList.subList(min, max);
                for (int y = 0; y < subList.size(); y++) {
                    line++;
                    RoomCodeImportDto roomCodeImportDto = (RoomCodeImportDto) subList.get(y);
                    roomCodeSet.add(roomCodeImportDto.getRoomCode());
                    roomNameSet.add(roomCodeImportDto.getRoomName());
                    map.put("dataRemark", "编码:" + roomCodeImportDto.getRoomCode() + ",名称:" + roomCodeImportDto.getRoomName());
                    dataVerify(examId, roomCodeImportDto.getTeacher1(), roomCodeImportDto.getTeacher1Pwd(), roomCodeImportDto.getRoomCode(), roomCodeImportDto.getRoomName(), orgId, createId, tbUserList, tbUserRoleList, tbExamInvigilateUserList);
                    map.put("dataRemark", "编码:" + roomCodeImportDto.getRoomCode() + ",名称:" + roomCodeImportDto.getRoomName());
                    dataVerify(examId, roomCodeImportDto.getTeacher2(), roomCodeImportDto.getTeacher2Pwd(), roomCodeImportDto.getRoomCode(), roomCodeImportDto.getRoomName(), orgId, createId, tbUserList, tbUserRoleList, tbExamInvigilateUserList);
                    map.put("dataRemark", "编码:" + roomCodeImportDto.getRoomCode() + ",名称:" + roomCodeImportDto.getRoomName());
                    dataVerify(examId, roomCodeImportDto.getTeacher3(), roomCodeImportDto.getTeacher3Pwd(), roomCodeImportDto.getRoomCode(), roomCodeImportDto.getRoomName(), orgId, createId, tbUserList, tbUserRoleList, tbExamInvigilateUserList);
                }
                txtList = this.importProgress(max, min, size, txtList, tbTaskHistory);
                if (max == size) {
                    break;
                }
                min = max;
                max += SystemConstant.MAX_IMPORT_SIZE;
                if (max >= size) {
                    max = size;
                }
            }
        }
        QueryWrapper<TBExamInvigilateUser> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(TBExamInvigilateUser::getOrgId, orgId)
                .in(TBExamInvigilateUser::getRoomCode, roomCodeSet)
                .eq(TBExamInvigilateUser::getExamId, examId)
                .in(TBExamInvigilateUser::getRoomName, roomNameSet);
        tbExamInvigilateUserService.remove(wrapper);

        if (!CollectionUtils.isEmpty(tbUserList)) {
            tbUserService.saveOrUpdateBatch(tbUserList);
        }
        if (!CollectionUtils.isEmpty(tbUserRoleList)) {
            tbUserRoleService.saveOrUpdateBatch(tbUserRoleList);
        }
        if (!CollectionUtils.isEmpty(tbExamInvigilateUserList)) {
            tbExamInvigilateUserService.saveOrUpdateBatch(tbExamInvigilateUserList);
        }
        map.put("line", line);
        return map;
    }

    /**
     * 考场导出逻辑
     *
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> execExportRoomCodeLogic(Map<String, Object> map) {
        return commonExport(map);
    }

    @Override
    @Transactional
    public Map<String, Object> execExportMarkResultSimpleLogic(Map<String, Object> map) {
        return commonExport(map);
    }

    @Override
    @Transactional
    public Map<String, Object> execExportMarkResultStandardLogic(Map<String, Object> map) {
        return commonExport(map);
    }

    /**
     * 考生导出
     *
     * @param map
     * @return
     */
    @Override
    public Map<String, Object> execExportExamStudentLogic(Map<String, Object> map) {
        return commonExport(map);
    }

    /**
     * 推送云阅卷数据逻辑
     *
     * @param map
     * @return
     */
    @Override
    public Map<String, Object> execPushCloudMarkDataLogic(Map<String, Object> map) throws IOException {
        checkData(map);
        Long examId = (Long) map.get(SystemConstant.EXAM_ID);
        Long orgId = (Long) map.get(SystemConstant.ORG_ID);
        String examCode = (String) map.get(SystemConstant.EXAM_CODE);

        //先推送考试
        ExamCacheBean examCacheBean = teExamService.getExamCacheBean(examId);
        TBTaskHistory tbTaskHistory = (TBTaskHistory) map.get("tbTaskHistory");
        JSONObject jsonObject = Objects.nonNull(tbTaskHistory.getRemark()) ? JSONObject.parseObject(tbTaskHistory.getRemark()) : null;
        Integer totalTaskSize = 1, currentTaskSize = 1, jsTotalTaskSize = 0, jsCurrentTaskSize = 0;
        Long cloudMarkExamId = null;

        TEExam teExam = teExamService.cacheConvert(examCacheBean);
        if (Objects.isNull(tbTaskHistory.getEntityId())) {
            cloudMarkExamId = cloudMarkUtil.callExamSaveApi(new SaveExamParams(orgId, examId, examCode, examCacheBean.getName(), DateUtil.format(new Date(examCacheBean.getStartTime()), Constants.DEFAULT_DATE_PATTERN)));
//            cloudMarkExamId = Long.parseLong((int) (Math.random() * 1000) + "");
            teExam.setUpdateTime(System.currentTimeMillis());
            teExam.setThirdExamId(String.valueOf(cloudMarkExamId));
            teExamService.updateById(teExam);
            teExamService.updateExamCacheBean(examId);
        } else {
            cloudMarkExamId = tbTaskHistory.getEntityId();
        }
        if (Objects.nonNull(cloudMarkExamId)) {
            if (Objects.isNull(jsonObject)) {
                jsonObject = new JSONObject();
                jsonObject.put("examSize", 1);
                jsonObject.put("totalTaskSize", totalTaskSize);
                jsonObject.put("currentTaskSize", currentTaskSize);
                tbTaskHistory.setEntityId(cloudMarkExamId);
                tbTaskHistory.setRemark(jsonObject.toJSONString());
            } else {
                jsTotalTaskSize = jsonObject.getInteger("totalTaskSize");
                jsCurrentTaskSize = jsonObject.getInteger("currentTaskSize");
                tbTaskHistory.setStatus(TaskStatusEnum.RUNNING);
            }
            tbTaskHistoryService.updateById(tbTaskHistory);

            //推送试卷科目
            Set<Long> examPaperIdSet = pushExamPaperLogic(map, jsonObject, totalTaskSize, jsCurrentTaskSize, currentTaskSize, cloudMarkExamId);
            if (CollectionUtils.isEmpty(examPaperIdSet)) {
                throw new BusinessException("没有考试试卷信息");
            }

            //获取所有待阅卷的考生
            List<TEExamStudent> teExamStudentList = getExamStudentNeedMarkAll(map, jsonObject, teExam, examPaperIdSet, jsTotalTaskSize, jsCurrentTaskSize);

            //获取只有一次考试记录的考生
            List<TOeExamRecord> tOeExamRecordList = getExamRecordOnce(map, jsonObject, examPaperIdSet);

            //获取多次考试记录的考试
            List<TOeExamRecord> tOeExamRecordManyList = tOeExamRecordService.findExamRecordNeedMarkAll(examId, examPaperIdSet, true);
            if (!CollectionUtils.isEmpty(tOeExamRecordManyList)) {
                Set<Long> manyExamStudentIdSet = tOeExamRecordManyList.stream().map(s -> s.getExamStudentId()).collect(Collectors.toSet());
                jsonObject.put("examRecordManyListSize", manyExamStudentIdSet.size());
                tbTaskHistory.setRemark(jsonObject.toJSONString());
                tbTaskHistoryService.updateById(tbTaskHistory);
                if (teExamStudentList.size() != tOeExamRecordList.size() + manyExamStudentIdSet.size()) {
                    throw new BusinessException("待阅卷的考试记录和考生数不匹配");
                }
                List<TOeExamRecord> resultExamRecordList = filterManyExamRecord(tOeExamRecordManyList, teExam);
                tOeExamRecordList.addAll(resultExamRecordList);
                //推送多次考试记录的考生
            } else {
                if (teExamStudentList.size() != tOeExamRecordList.size()) {
                    throw new BusinessException("待阅卷的考试记录和考生数不匹配");
                }
            }

            //推送考试记录的考生开始
            pushExamStudentLogic(map,
                    jsonObject,
                    teExamStudentList,
                    tOeExamRecordList,
                    jsCurrentTaskSize,
                    cloudMarkExamId);
        }
        return pushDataSuccess(map, jsonObject);
    }

    /**
     * 推送数据成功
     *
     * @param map
     * @param jsonObject
     * @return
     */
    private Map<String, Object> pushDataSuccess(Map<String, Object> map,
                                                JSONObject jsonObject) {
        TBTaskHistory tbTaskHistory = (TBTaskHistory) map.get("tbTaskHistory");
        List<String> txtList = (List<String>) map.get("txtList");
        StringJoiner stringJoiner = new StringJoiner("");
        stringJoiner.add("推送数据成功：共推送了").add(String.valueOf(jsonObject.getInteger("examSize"))).add("个考试，")
                .add(String.valueOf(jsonObject.getInteger("examPaperSize"))).add("个试卷科目，")
                .add(String.valueOf(jsonObject.getInteger("examExamStudentSize"))).add("个考生");
        txtList.add(DateUtil.format(new Date(), Constants.DEFAULT_DATE_PATTERN) + "->" + stringJoiner.toString());

        tbTaskHistory.setSummary(txtList.toString());
        tbTaskHistory.setStatus(TaskStatusEnum.FINISH);
        tbTaskHistory.setFinishTime(System.currentTimeMillis());
        tbTaskHistoryService.updateById(tbTaskHistory);
        return map;
    }

    /**
     * 获取只有一次考试记录的考生
     *
     * @param map
     * @param jsonObject
     * @param examPaperIdSet
     * @return
     */
    private List<TOeExamRecord> getExamRecordOnce(Map<String, Object> map,
                                                  JSONObject jsonObject,
                                                  Set<Long> examPaperIdSet) {
        Long examId = (Long) map.get(SystemConstant.EXAM_ID);
        TBTaskHistory tbTaskHistory = (TBTaskHistory) map.get("tbTaskHistory");
        List<TOeExamRecord> tOeExamRecordList = tOeExamRecordService.findExamRecordNeedMarkAll(examId, examPaperIdSet, false);
        if (CollectionUtils.isEmpty(tOeExamRecordList)) {
            throw new BusinessException("没有待阅卷的考试记录");
        }
        jsonObject.put("examRecordListSize", tOeExamRecordList.size());

        Integer totalTaskSize = jsonObject.getInteger("totalTaskSize");
        Integer currentTaskSize = jsonObject.getInteger("currentTaskSize");

        BigDecimal progress = new BigDecimal(0);
        BigDecimal b = new BigDecimal(100);
        progress = new BigDecimal(currentTaskSize).divide(new BigDecimal(totalTaskSize), 2, BigDecimal.ROUND_HALF_UP).multiply(b);
        if (Objects.isNull(tbTaskHistory.getProgress()) || (Objects.nonNull(tbTaskHistory.getProgress()) && tbTaskHistory.getProgress().doubleValue() < progress.doubleValue())) {
            tbTaskHistory.setProgress(progress.doubleValue());
            tbTaskHistoryService.updateById(tbTaskHistory);
        }
        tbTaskHistory.setRemark(jsonObject.toJSONString());
        return tOeExamRecordList;
    }

    /**
     * 获取所有待阅卷的考生
     *
     * @param map
     * @param jsonObject
     * @param teExam
     * @param examPaperIdSet
     * @param jsTotalTaskSize
     * @param jsCurrentTaskSize
     * @return
     */
    private List<TEExamStudent> getExamStudentNeedMarkAll(Map<String, Object> map,
                                                          JSONObject jsonObject,
                                                          TEExam teExam,
                                                          Set<Long> examPaperIdSet,
                                                          Integer jsTotalTaskSize,
                                                          Integer jsCurrentTaskSize) {
        Long examId = (Long) map.get(SystemConstant.EXAM_ID);
        TBTaskHistory tbTaskHistory = (TBTaskHistory) map.get("tbTaskHistory");
        List<TEExamStudent> teExamStudentList = teExamStudentService.findExamStudentNeedMarkAll(examId, examPaperIdSet);
        if (CollectionUtils.isEmpty(teExamStudentList)) {
            throw new BusinessException("没有待阅卷的考生");
        }
        Integer totalTaskSize = jsonObject.getInteger("totalTaskSize");
        Integer currentTaskSize = jsonObject.getInteger("currentTaskSize");
        totalTaskSize = totalTaskSize.intValue() + teExamStudentList.size();
        if (jsCurrentTaskSize.intValue() > currentTaskSize.intValue() && jsTotalTaskSize.intValue() > 0 && jsTotalTaskSize.intValue() != totalTaskSize.intValue()) {
            teExam.setUpdateTime(System.currentTimeMillis());
            teExam.setThirdExamId(null);
            teExamService.updateById(teExam);
            teExamService.updateExamCacheBean(examId);
            throw new BusinessException("数据已发生改变，需重新推送");
        }
        jsonObject.put("examExamStudentSize", teExamStudentList.size());
        jsonObject.put("totalTaskSize", totalTaskSize);
        tbTaskHistory.setRemark(jsonObject.toJSONString());
        return teExamStudentList;
    }

    /**
     * 推送考生和考试记录逻辑
     *
     * @param map
     * @param jsonObject
     * @param teExamStudentList
     * @param tOeExamRecordList
     * @param jsCurrentTaskSize
     * @param cloudMarkExamId
     */
    private void pushExamStudentLogic(Map<String, Object> map,
                                      JSONObject jsonObject,
                                      List<TEExamStudent> teExamStudentList,
                                      List<TOeExamRecord> tOeExamRecordList,
                                      Integer jsCurrentTaskSize,
                                      Long cloudMarkExamId) throws IOException {
        Long orgId = (Long) map.get(SystemConstant.ORG_ID);
        TBTaskHistory tbTaskHistory = (TBTaskHistory) map.get("tbTaskHistory");
        BigDecimal progress = new BigDecimal(0);
        BigDecimal b = new BigDecimal(100);
        TBOrg tbOrg = cacheService.addOrgCache(orgId);

        Integer totalTaskSize = jsonObject.getInteger("totalTaskSize");
        Integer currentTaskSize = jsonObject.getInteger("currentTaskSize");

        Map<Long, TOeExamRecord> tOeExamRecordMap = tOeExamRecordList.stream().collect(
                Collectors.toMap(TOeExamRecord::getExamStudentId, Function.identity(), (dto1, dto2) -> dto1));

        Map<Long, Map<String, Integer>> paperStructMap = new HashMap<>();
        List<File> recordJsonList = new ArrayList<>(teExamStudentList.size());
        map.put("recordJsonList", recordJsonList);
        //推送考试记录的考生
        int min = 0;
        int max = SystemConstant.MAX_IMPORT_SIZE, size = teExamStudentList.size();
        if (max >= size) {
            max = size;
        }
        while (max <= size) {
            List<TEExamStudent> subList = teExamStudentList.subList(min, max);
            for (int y = 0; y < subList.size(); y++) {
                TEExamStudent t = subList.get(y);
                if (jsCurrentTaskSize.intValue() > 0 && currentTaskSize.intValue() < jsCurrentTaskSize.intValue()) {
                    jsonObject.put("currentTaskSize", currentTaskSize);
                    currentTaskSize++;
                    tbTaskHistory.setRemark(jsonObject.toJSONString());
                    continue;
                } else {
                    cloudMarkUtil.callStudentSaveApi(new SaveStudentParams(orgId, cloudMarkExamId, SystemConstant.VALUE_OF_BLANK_REQUIRED_FIELD,
                            t.getIdentity(),
                            t.getName(),
                            tbOrg.getName(),
                            t.getClassNo(), SystemConstant.VALUE_OF_BLANK_REQUIRED_FIELD,
                            t.getCourseCode(),
                            t.getCourseName()));
                    TOeExamRecord tOeExamRecord = tOeExamRecordMap.get(t.getId());
                    Optional.ofNullable(tOeExamRecord).orElseThrow(() -> new BusinessException(ExceptionResultEnum.NOT_FOUND_EXAM_RECORD));

                    Gson gson = new Gson();
                    OpenRecordNeedMarkBean openRecordNeedMarkBean = gson.fromJson(gson.toJson(tOeExamRecord), OpenRecordNeedMarkBean.class);
                    List<OpenRecordAnswerTempBean> answersTemp = examAnswerService.findByExamRecordId(tOeExamRecord.getId());
                    List<OpenRecordAnswerBean> answers = new ArrayList<>(answersTemp.size());
                    List<OpenRecordAnswerBean> finalAnswers = answers;
                    answersTemp.forEach(s -> {
                        OpenRecordAnswerBean openRecordAnswerBean = new OpenRecordAnswerBean(s.getMainNumber(), s.getSubNumber(), s.getSubIndex());
                        try {
                            openRecordAnswerBean.setAnswer(gson.fromJson(s.getAnswer(), new TypeToken<List<OpenRecordAnswerStructBean>>() {
                            }.getType()));
                        } catch (Exception e) {
                            log.error("错误答案:{}", s.getAnswer(), e);
                        }
                        finalAnswers.add(openRecordAnswerBean);
                    });
                    openRecordNeedMarkBean.setAnswers(answers.stream().filter(answer -> {
                        Map<String, Integer> struct = paperStructMap.get(tOeExamRecord.getPaperId());
                        if (struct == null) {
                            struct = teExamPaperService.getPaperStructCacheBean(tOeExamRecord.getPaperId());
                            if (struct != null) {
                                paperStructMap.put(tOeExamRecord.getPaperId(), struct);
                            } else {
                                log.error("找不到对应的试卷结构，paperId=" + tOeExamRecord.getPaperId());
                                return false;
                            }
                        }
                        Integer type = struct.get(RedisKeyHelper
                                .examAnswerHashKey(answer.getMainNumber(), answer.getSubNumber(),
                                        answer.getSubIndex()));
                        //过滤单选、多选、判断题
                        return type != null && type != 1 && type != 2 && type != 3;
                    }).collect(Collectors.toList()));

                    File fileAnswerJson = new File(SystemConstant.TEMP_FILES_DIR + File.separator + CloudMarkFileUploadTypeEnum.ANSWER.getCode() + File.separator + SystemConstant.getUuid() + ".json");
                    if (!fileAnswerJson.exists()) {
                        fileAnswerJson.getParentFile().mkdirs();
                        fileAnswerJson.createNewFile();
                    }
                    IOUtils.write(JacksonUtil.parseJson(openRecordNeedMarkBean).getBytes(SystemConstant.CHARSET_NAME), new FileOutputStream(fileAnswerJson));
                    recordJsonList.add(fileAnswerJson);
                    cloudMarkUtil.callFileUploadApi(new FileUploadParams(orgId, cloudMarkExamId, String.valueOf(tOeExamRecord.getId()), fileAnswerJson));

                    jsonObject.put("currentTaskSize", currentTaskSize);
                    currentTaskSize++;
                    tbTaskHistory.setRemark(jsonObject.toJSONString());
                }
            }
            progress = new BigDecimal(currentTaskSize).divide(new BigDecimal(totalTaskSize), 2, BigDecimal.ROUND_HALF_UP).multiply(b);
            if (Objects.isNull(tbTaskHistory.getProgress()) || (Objects.nonNull(tbTaskHistory.getProgress()) && tbTaskHistory.getProgress().doubleValue() < progress.doubleValue())) {
                tbTaskHistory.setProgress(progress.doubleValue());
                tbTaskHistoryService.updateById(tbTaskHistory);
            }
            if (max == size) {
                break;
            }
            min = max;
            max += SystemConstant.MAX_IMPORT_SIZE;
            if (max >= size) {
                max = size;
            }
        }
        jsonObject.put("currentTaskSize", currentTaskSize);
        tbTaskHistory.setRemark(jsonObject.toJSONString());
    }

    /**
     * 过滤多条考试记录的考生
     *
     * @param tOeExamRecordManyList
     * @param teExam
     * @return
     */
    private List<TOeExamRecord> filterManyExamRecord(List<TOeExamRecord> tOeExamRecordManyList, TEExam teExam) {
        List<TOeExamRecord> result = null;
        LinkedMultiValueMap<Long, TOeExamRecord> tOeExamRecordLinkedMultiValueMap = new LinkedMultiValueMap<>();
        for (TOeExamRecord bean : tOeExamRecordManyList) {
            tOeExamRecordLinkedMultiValueMap.add(bean.getExamStudentId(), bean);
        }
        if (!CollectionUtils.isEmpty(tOeExamRecordLinkedMultiValueMap)) {
            result = new ArrayList<>();
            List<TOeExamRecord> finalResult = result;
            tOeExamRecordLinkedMultiValueMap.forEach((k, v) -> {
                double highestObjectiveScore = 0.0;
                long maxFinishTime = 0L;
                boolean subjective = false;
                List<TOeExamRecord> tempResult = new ArrayList<>(v.size());
                for (TOeExamRecord bean : v) {
                    //获取对应的试卷信息
                    ExamPaperCacheBean paper = teExamPaperService.getExamPaperCacheBean(bean.getPaperId());
                    if (Objects.isNull(paper)) {
                        log.error("找不到对应的试卷信息, paperId=" + bean.getPaperId());
                        continue;
                    }
                    //根据试卷结构过滤没有主观题的情况
                    if (paper.getTotalSubjectiveScore().doubleValue() == 0D) {
                        subjective = true;
                        maxFinishTime = Math.max(maxFinishTime, bean.getFinishTime());
                        continue;
                    }
                    highestObjectiveScore = Math.max(highestObjectiveScore, bean.getObjectiveScore());
                    maxFinishTime = Math.max(maxFinishTime, bean.getFinishTime());
                    tempResult.add(bean);
                }
                if (tempResult.isEmpty()) {
                    return;
                }
                switch (teExam.getRecordSelectStrategy()) {
                    case HIGHEST_OBJECTIVE_SCORE:
                        if (subjective) {
                            final long time = maxFinishTime;
                            finalResult.addAll(v.stream().filter(bean -> bean.getFinishTime() == time).collect(Collectors.toList()));
                        } else {
                            //客观分最高，客观分相同则全部提交阅卷
                            final double score = highestObjectiveScore;
                            finalResult.addAll(tempResult.stream().filter(bean -> bean.getObjectiveScore() == score).collect(Collectors.toList()));
                        }
                        break;
                    case LATEST:
                        //最后一次提交
                        final long time = maxFinishTime;
                        finalResult.addAll(tempResult.stream().filter(bean -> bean.getFinishTime() == time).collect(Collectors.toList()));
                        break;
                    default:
                        //总分最高，全部提交阅卷
                        finalResult.addAll(tempResult);
                        break;
                }
            });
        }
        return result;
    }

    /**
     * 推送试卷逻辑
     *
     * @param map
     * @param jsonObject
     * @param totalTaskSize
     * @param jsCurrentTaskSize
     * @param currentTaskSize
     * @param cloudMarkExamId
     * @return
     * @throws IOException
     */
    private Set<Long> pushExamPaperLogic(Map<String, Object> map,
                                         JSONObject jsonObject,
                                         Integer totalTaskSize,
                                         Integer jsCurrentTaskSize,
                                         Integer currentTaskSize,
                                         Long cloudMarkExamId) throws IOException {
        Long examId = (Long) map.get(SystemConstant.EXAM_ID);
        Long orgId = (Long) map.get(SystemConstant.ORG_ID);
        TBTaskHistory tbTaskHistory = (TBTaskHistory) map.get("tbTaskHistory");
        List<File> paperJsonList = null;

        QueryWrapper<TEExamPaper> teExamPaperQueryWrapper = new QueryWrapper<>();
        teExamPaperQueryWrapper.lambda().eq(TEExamPaper::getExamId, examId)
                .gt(TEExamPaper::getTotalSubjectiveScore, 0).orderByAsc(TEExamPaper::getId);
        List<TEExamPaper> teExamPaperList = teExamPaperService.list(teExamPaperQueryWrapper);
        if (CollectionUtils.isEmpty(teExamPaperList)) {
            throw new BusinessException("没有考试试卷");
        }
        totalTaskSize = totalTaskSize.intValue() + teExamPaperList.size();
        jsonObject.put("totalTaskSize", totalTaskSize);
        jsonObject.put("examPaperSize", teExamPaperList.size());
        tbTaskHistory.setRemark(jsonObject.toJSONString());
        Set<Long> examPaperIdSet = null;

        if (!CollectionUtils.isEmpty(teExamPaperList)) {
            paperJsonList = new ArrayList<>(teExamPaperList.size());
            map.put("paperJsonList", paperJsonList);
            examPaperIdSet = new HashSet<>(teExamPaperList.size());

            BigDecimal progress = new BigDecimal(0);
            BigDecimal b = new BigDecimal(100);

            int min = 0;
            int max = SystemConstant.MAX_IMPORT_SIZE, size = teExamPaperList.size();
            if (max >= size) {
                max = size;
            }
            while (max <= size) {
                List<TEExamPaper> subList = teExamPaperList.subList(min, max);
                for (int y = 0; y < subList.size(); y++) {
                    TEExamPaper t = subList.get(y);
                    examPaperIdSet.add(t.getId());
                    if (jsCurrentTaskSize.intValue() > 0 && currentTaskSize.intValue() < jsCurrentTaskSize.intValue()) {
                        jsonObject.put("currentTaskSize", currentTaskSize);
                        currentTaskSize++;
                        tbTaskHistory.setRemark(jsonObject.toJSONString());
                        continue;
                    } else {
                        String courseCode = t.getCourseCode() + SystemConstant.JOINT_MARK + t.getId();
                        cloudMarkUtil.callSubjectSaveApi(new SaveSubjectParams(orgId, cloudMarkExamId, courseCode, t.getName()));
                        //解析试卷JSON文件
                        JSONObject paperResult = JSONObject.parseObject(new String(ossUtil.download(false, t.getPaperViewPath()), StandardCharsets.UTF_8));
                        paperResult.put(SystemConstant.ID, t.getId());
                        paperResult.put("code", t.getCode());
                        paperResult.put(SystemConstant.NAME, t.getName());
                        if (Objects.nonNull(t.getAnswerPath())) {
                            JSONArray detailCollection = new JSONArray();
                            JSONArray paperDetails = paperResult.getJSONArray("details");
                            //解析答案JSON文件
                            JSONObject answerJson = JSONObject.parseObject(new String(ossUtil.download(false, t.getAnswerPath()), StandardCharsets.UTF_8));
                            JSONArray answerDetails = answerJson.getJSONArray("details");
                            for (int j = 0; j < paperDetails.size(); j++) {
                                //遍历所有大题
                                JSONObject paperDetail = paperDetails.getJSONObject(j);
                                JSONObject answerDetail = findJsonObject(answerDetails, paperDetail.getIntValue("number"));
                                //按条件过滤需要的小题同时合并答案
                                JSONArray questionCollection = filterQuestions(paperDetail.getJSONArray("questions"),
                                        Objects.nonNull(answerDetail) ? answerDetail.getJSONArray("questions") : null, true,
                                        true);
                                //有小题的情况下，本大题才需要被包含
                                if (questionCollection.size() > 0) {
                                    paperDetail.put("questions", questionCollection);
                                    detailCollection.add(paperDetail);
                                }
                            }
                            paperResult.put("details", detailCollection);
                        }

                        File filePaperJson = new File(SystemConstant.TEMP_FILES_DIR + File.separator + CloudMarkFileUploadTypeEnum.PAPER.getCode() + File.separator + SystemConstant.getUuid() + ".json");
                        if (!filePaperJson.exists()) {
                            filePaperJson.getParentFile().mkdirs();
                            filePaperJson.createNewFile();
                        }
                        IOUtils.write(paperResult.toJSONString().getBytes(SystemConstant.CHARSET_NAME), new FileOutputStream(filePaperJson));
                        paperJsonList.add(filePaperJson);
                        FileUploadParams fileUploadParamsPaperJson = new FileUploadParams(orgId, cloudMarkExamId, courseCode, CloudMarkFileUploadTypeEnum.PAPER, filePaperJson);
                        fileUploadParamsPaperJson.setFormat(CloudMarkFileUploadTypeEnum.JSON.getCode());
                        cloudMarkUtil.callFileUploadApi(fileUploadParamsPaperJson);

                        jsonObject.put("currentTaskSize", currentTaskSize);
                        currentTaskSize++;
                        tbTaskHistory.setRemark(jsonObject.toJSONString());
                    }
                }
                progress = new BigDecimal(currentTaskSize).divide(new BigDecimal(totalTaskSize), 2, BigDecimal.ROUND_HALF_UP).multiply(b);
                if (Objects.isNull(tbTaskHistory.getProgress()) || (Objects.nonNull(tbTaskHistory.getProgress()) && tbTaskHistory.getProgress().doubleValue() < progress.doubleValue())) {
                    tbTaskHistory.setProgress(progress.doubleValue());
                    tbTaskHistoryService.updateById(tbTaskHistory);
                }
                if (max == size) {
                    break;
                }
                min = max;
                max += SystemConstant.MAX_IMPORT_SIZE;
                if (max >= size) {
                    max = size;
                }
            }
            jsonObject.put("currentTaskSize", currentTaskSize);
            tbTaskHistory.setRemark(jsonObject.toJSONString());
            tbTaskHistoryService.updateById(tbTaskHistory);
        }
        return examPaperIdSet;
    }

    /**
     * 数据检查
     *
     * @param map
     */
    private void checkData(Map<String, Object> map) {
        Long examId = (Long) map.get(SystemConstant.EXAM_ID);
        if (Objects.isNull(examId)) {
            throw new RuntimeException("考试id为空");
        }
        Long orgId = (Long) map.get(SystemConstant.ORG_ID);
        if (Objects.isNull(orgId)) {
            throw new BusinessException("机构id为空");
        }
        String examCode = (String) map.get(SystemConstant.EXAM_CODE);
        if (Objects.isNull(examCode)) {
            throw new RuntimeException("考试编码为空");
        }
        TBOrg tbOrg = cacheService.addOrgCache(orgId);
        Optional.ofNullable(tbOrg).orElseThrow(() -> new BusinessException(ExceptionResultEnum.ORG_NO));

        //先推送考试
        ExamCacheBean examCacheBean = teExamService.getExamCacheBean(examId);
        Optional.ofNullable(examCacheBean).orElseThrow(() -> new BusinessException(ExceptionResultEnum.EXAM_NO));

        if (Objects.isNull(examCacheBean.getEnable()) || examCacheBean.getEnable().intValue() == 0) {
            throw new BusinessException("考试批次已禁用");
        }
    }

    /**
     * 过滤答案
     *
     * @param array
     * @param number
     * @return
     */
    private JSONObject findJsonObject(JSONArray array, int number) {
        if (array != null) {
            for (int i = 0; i < array.size(); i++) {
                JSONObject item = array.getJSONObject(i);
                if (item.getIntValue("number") == number) {
                    return item;
                }
            }
        }
        return null;
    }

    /**
     * 过滤题目
     *
     * @param questions
     * @param answers
     * @param acceptObjective
     * @param acceptSubjective
     * @return
     */
    private JSONArray filterQuestions(JSONArray questions, JSONArray answers, boolean acceptObjective,
                                      boolean acceptSubjective) {
        JSONArray collection = new JSONArray();
        for (int i = 0; i < questions.size(); i++) {
            JSONObject question = questions.getJSONObject(i);
            //根据题号查找答案
            JSONObject answer = findJsonObject(answers, question.getIntValue("number"));
            boolean accept = true;
            //判断结构类型
            switch (question.getIntValue("structType")) {
                case 1:
                    //单选
                case 2:
                    //多选
                case 3:
                    //判断
                    accept = acceptObjective;
                    break;
                case 4:
                    //填空
                case 5:
                    //问答
                    accept = acceptSubjective;
                    break;
                case 6:
                    //套题
                    //按条件过滤套题下所有子题
                    JSONArray subAnswers = answer != null ? answer.getJSONArray("subQuestions") : null;
                    JSONArray subCollection = filterQuestions(question.getJSONArray("subQuestions"), subAnswers,
                            acceptObjective, acceptSubjective);
                    //没有子题，则本题也被过滤掉
                    if (subCollection.size() > 0) {
                        question.put("subQuestions", subCollection);
                    } else {
                        accept = false;
                    }
                    break;
                default:
                    accept = false;
            }
            if (accept) {
                if (answer != null) {
                    question.put("answer", answer.get("answer"));
                }
                collection.add(question);
            }
        }
        return collection;
    }

    /**
     * 查询监考帐号是否存在
     *
     * @param examId
     * @param userName
     * @param password
     * @param roomCode
     * @param roomName
     * @param orgId
     * @param createId
     */
    public void dataVerify(Long examId, String userName, String password, String roomCode, String roomName, Long orgId, Long createId, Set<TBUser> tbUserList, Set<TBUserRole> tbUserRoleList, Set<TBExamInvigilateUser> tbExamInvigilateUserList) throws UnsupportedEncodingException {
        if (!checkRoomCodeExist(examId, roomCode, orgId, roomName)) {
            throw new BusinessException("考场代码 " + roomCode + " 不存在");
        }
        if (Objects.nonNull(userName)) {
            //先查询监考帐号是否存在
            QueryWrapper<TBUser> tbUserQueryWrapper = new QueryWrapper<>();
            tbUserQueryWrapper.lambda().eq(TBUser::getLoginName, userName).eq(TBUser::getOrgId, orgId);
            TBUser tbUser = tbUserService.getOne(tbUserQueryWrapper);
            if (Objects.isNull(tbUser)) {
                //新增用户
                tbUser = new TBUser(orgId, userName, userName, createId, password);
                TBUserRole tbUserRole = new TBUserRole(tbUser.getId(), RoleEnum.INVIGILATE.name());
                tbUserRoleList.add(tbUserRole);
            } else {
                if (Objects.nonNull(password) && !Objects.equals(password, "")) {
                    tbUser.setPassword(Base64Util.encode(password.getBytes(SystemConstant.CHARSET_NAME)));
                } else {
                    tbUser.setPassword(SystemConstant.DEFAULT_PASSWORD);
                }
            }
            tbUserList.add(tbUser);

            QueryWrapper<TBExamInvigilateUser> tbExamInvigilateUserQueryWrapper = new QueryWrapper<>();
            tbExamInvigilateUserQueryWrapper.lambda().eq(TBExamInvigilateUser::getOrgId, orgId)
                    .eq(TBExamInvigilateUser::getUserId, tbUser.getId())
                    .eq(TBExamInvigilateUser::getRoomCode, roomCode)
                    .eq(TBExamInvigilateUser::getExamId, examId)
                    .eq(TBExamInvigilateUser::getRoomName, roomName);
            TBExamInvigilateUser tbExamInvigilateUser = tbExamInvigilateUserService.getOne(tbExamInvigilateUserQueryWrapper);
            if (Objects.isNull(tbExamInvigilateUser)) {//新增考场
                tbExamInvigilateUser = new TBExamInvigilateUser(examId, orgId, tbUser.getId(), roomCode, roomName);
            }
            tbExamInvigilateUserList.add(tbExamInvigilateUser);
        } else {
            TBExamInvigilateUser tbExamInvigilateUser = new TBExamInvigilateUser(examId, orgId, null, roomCode, roomName);
            if (!tbExamInvigilateUserList.contains(tbExamInvigilateUser)) {
                tbExamInvigilateUserList.add(tbExamInvigilateUser);
            }
        }
    }

    private boolean checkRoomCodeExist(Long examId, String roomCode, Long orgId, String roomName) {
        QueryWrapper<TBExamInvigilateUserTemp> tbExamInvigilateUserTempQueryWrapper = new QueryWrapper<>();
        tbExamInvigilateUserTempQueryWrapper.lambda().eq(TBExamInvigilateUserTemp::getExamId, examId)
                .eq(TBExamInvigilateUserTemp::getOrgId, orgId)
                .eq(TBExamInvigilateUserTemp::getRoomCode, roomCode)
                .eq(TBExamInvigilateUserTemp::getRoomName, roomName);
        TBExamInvigilateUserTemp tbExamInvigilateUserTemp = tbExamInvigilateUserTempService.getOne(tbExamInvigilateUserTempQueryWrapper);
        tbExamInvigilateUserTemp = Objects.isNull(tbExamInvigilateUserTemp) ? new TBExamInvigilateUserTemp(orgId, examId, roomCode, roomName) : tbExamInvigilateUserTemp;
        return tbExamInvigilateUserTempService.saveOrUpdate(tbExamInvigilateUserTemp);
    }

    /**
     * 导入换算进度
     *
     * @param max
     * @param min
     * @param size
     * @param txtList
     * @param tbTaskHistory
     * @return
     */
    public List<String> importProgress(int max, int min, int size, List<String> txtList, TBTaskHistory tbTaskHistory) {
        BigDecimal bigDecimal = new BigDecimal(100);
        BigDecimal progress = new BigDecimal(Double.valueOf(new BigDecimal(max).divide(new BigDecimal(size), 2, BigDecimal.ROUND_HALF_UP).multiply(bigDecimal).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue())).setScale(0, BigDecimal.ROUND_HALF_UP);
        if (progress.intValue() == 100) {
            tbTaskHistory.setStatus(TaskStatusEnum.FINISH);
            tbTaskHistory.setSummary("共处理了" + size + "条数据");
            tbTaskHistory.setFinishTime(System.currentTimeMillis());
            txtList.add(DateUtil.format(new Date(), Constants.DEFAULT_DATE_PATTERN) + "->数据处理完毕，" + tbTaskHistory.getSummary());
        } else {
            tbTaskHistory.setStatus(TaskStatusEnum.RUNNING);
            tbTaskHistory.setSummary("正在处理第" + min + "条至" + max + "条数据");
            txtList.add(DateUtil.format(new Date(), Constants.DEFAULT_DATE_PATTERN) + "->数据处理中，" + tbTaskHistory.getSummary());
        }
        tbTaskHistory.setProgress(progress.doubleValue());
        return txtList;
    }

    /**
     * 导出换算进度
     *
     * @param max
     * @param min
     * @param size
     * @param txtList
     * @param tbTaskHistory
     * @return
     */
    public List<String> exportProgress(int max, int min, int size, List<String> txtList, TBTaskHistory tbTaskHistory) {
        BigDecimal bigDecimal = new BigDecimal(100);
        BigDecimal progress = null;
        if (size > 0) {
            progress = new BigDecimal(Double.valueOf(new BigDecimal(max).divide(new BigDecimal(size), 2, BigDecimal.ROUND_HALF_UP).multiply(bigDecimal).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue())).setScale(0, BigDecimal.ROUND_HALF_UP);
        } else {
            progress = new BigDecimal(-1);
        }
        if (progress.intValue() == 100) {
            tbTaskHistory.setStatus(TaskStatusEnum.FINISH);
            tbTaskHistory.setSummary("共导出了" + size + "条数据");
            tbTaskHistory.setFinishTime(System.currentTimeMillis());
            txtList.add(DateUtil.format(new Date(), Constants.DEFAULT_DATE_PATTERN) + "->数据导出完毕，" + tbTaskHistory.getSummary());
        } else if (progress.intValue() == -1) {
            tbTaskHistory.setStatus(TaskStatusEnum.FINISH);
            tbTaskHistory.setSummary("没有导出数据");
            tbTaskHistory.setFinishTime(System.currentTimeMillis());
            txtList.add(DateUtil.format(new Date(), Constants.DEFAULT_DATE_PATTERN) + "->数据导出完毕，" + tbTaskHistory.getSummary());
        } else {
            tbTaskHistory.setStatus(TaskStatusEnum.RUNNING);
            tbTaskHistory.setSummary("正在导出第" + min + "条至" + max + "条数据");
            txtList.add(DateUtil.format(new Date(), Constants.DEFAULT_DATE_PATTERN) + "->数据导出中，" + tbTaskHistory.getSummary());
        }
        tbTaskHistory.setProgress(progress.doubleValue());
        return txtList;
    }

    /**
     * 导入公用
     *
     * @param map
     * @return
     */
    private Map<String, Object> commonExport(Map<String, Object> map) {
        List<String> txtList = (List<String>) map.get("txtList");
        TBTaskHistory tbTaskHistory = (TBTaskHistory) map.get("tbTaskHistory");
        Integer max = Integer.parseInt(String.valueOf(map.get("max")));
        Integer min = Integer.parseInt(String.valueOf(map.get("min")));
        Integer size = Integer.parseInt(String.valueOf(map.get("size")));
        this.exportProgress(max, min, size, txtList, tbTaskHistory);
        return map;
    }
}